<?php

namespace IGD;

defined( 'ABSPATH' ) || exit;

class Admin {
	/**
	 * @var null
	 */
	protected static $instance = null;

	private $pages = [];

	public function __construct() {
		add_action( 'admin_menu', [ $this, 'admin_menu' ] );

		add_action( 'admin_notices', [ $this, 'lost_authorization_notice' ] );

		add_action( 'admin_init', [ $this, 'init_update' ] );

		//Handle custom app authorization
		add_action( 'admin_init', [ $this, 'app_authorization' ] );

		// Remove admin notices from plugin pages
		add_action( 'admin_notices', [ $this, 'display_notices' ] );

		//admin body class
		add_filter( 'admin_body_class', [ $this, 'admin_body_class' ] );

	}

	public function admin_body_class( $classes ) {
		$admin_pages = Admin::instance()->get_pages();

		global $current_screen;

		if ( in_array( $current_screen->id, $admin_pages ) ) {
			$key = array_search( $current_screen->id, $admin_pages );

			$classes .= ' igd-admin-page igd_' . $key . ' ';
		}

		return $classes;
	}

	public function display_notices() {

		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if ( get_option( 'igd_account_notice' ) ) {
			ob_start();
			include IGD_INCLUDES . '/views/notice/account.php';
			$notice_html = ob_get_clean();
			igd()->add_notice( 'info igd-account-notice error', $notice_html );

			return;
		}

		//Rating notice
		if ( 'off' != get_option( 'igd_rating_notice' ) && 'off' != get_transient( 'igd_rating_notice_interval' ) ) {

			ob_start();
			include IGD_INCLUDES . '/views/notice/rating.php';
			$notice_html = ob_get_clean();
			igd()->add_notice( 'info igd-rating-notice', $notice_html );

			return;
		}
	}

	public function app_authorization() {

		if ( isset( $_GET['action'] ) && 'integrate-google-drive-authorization' == sanitize_key( $_GET['action'] ) ) {

			$state     = $_GET['state'];
			$state_url = base64_decode( $state );

			//remove action from params
			unset( $_GET['action'] );

			$params       = http_build_query( $_GET );
			$redirect_url = $state_url . '&' . $params;

			wp_redirect( $redirect_url );

			exit();
		}
	}

	public function init_update() {

		if ( current_user_can( 'manage_options' ) ) {
			include_once IGD_INCLUDES . '/class-update.php';

			$updater = Update::instance();
			if ( $updater->needs_update() ) {
				$updater->perform_updates();
			}
		}
	}

	public function admin_menu() {

		$main_menu_added = false;
		$current_user    = wp_get_current_user();

		$access_rights = [
			'file_browser'      => [
				'view'          => [ 'IGD\App', 'view' ],
				'title'         => __( 'File Browser - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'File Browser', 'integrate-google-drive' )
			],
			'shortcode_builder' => [
				'view'          => [ 'IGD\Shortcode_Builder', 'view' ],
				'title'         => __( 'Shortcode Builder - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'Shortcode Builder', 'integrate-google-drive' )
			],
			'private_files'     => [
				'view'          => [ 'IGD\Private_Folders', 'view' ],
				'title'         => __( 'Users Private Files - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'Users Private Files', 'integrate-google-drive' )
			],
			'getting_started'   => [
				'view'          => [ $this, 'render_getting_started_page' ],
				'title'         => __( 'Getting Started - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'Getting Started', 'integrate-google-drive' )
			],
			'statistics'        => [
				'view'          => [ 'IGD\Statistics', 'view' ],
				'title'         => __( 'Statistics - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'Statistics', 'integrate-google-drive' )
			],
			'settings'          => [
				'view'          => [ $this, 'render_settings_page' ],
				'title'         => __( 'Settings - Integrate Google Drive', 'integrate-google-drive' ),
				'submenu_title' => __( 'Settings', 'integrate-google-drive' )
			]
		];

		if ( ! igd_fs()->can_use_premium_code__premium_only() || !igd_get_settings('enableStatistics', false) ) {
			unset( $access_rights['statistics'] );
		}

		foreach ( $access_rights as $access_right => $page_config ) {

			$access_users = igd_get_settings( "access" .str_replace(' ', '', ucwords(str_replace('_', ' ', $access_right))) . "Users", [ 'administrator' ] );
			$user_roles   = array_filter( $access_users, 'is_string' );
			$can_access   = ! empty( array_intersect( $current_user->roles, $user_roles ) ) || in_array( $current_user->ID, $access_users ) || ( is_multisite() && is_super_admin() );

			if ( $can_access ) {
				if ( ! $main_menu_added ) {
					$this->pages[ $access_right . '_page' ] = $this->add_main_menu_page( $page_config['title'], $page_config['submenu_title'], $page_config['view'] );
					$main_menu_added                        = true;
				} else {
					$this->pages[ $access_right . '_page' ] = $this->add_submenu_page( $page_config['title'], $page_config['submenu_title'], $page_config['view'], $access_right );
				}
			}
		}

		//Recommended plugins page
		if ( empty( get_option( "igd_hide_recommended_plugins" ) ) ) {
			add_submenu_page(
				'integrate-google-drive',
				esc_html__( 'Recommended Plugins', 'integrate-google-drive' ),
				esc_html__( 'Recommended Plugins', 'integrate-google-drive' ),
				'manage_options',
				'integrate-google-drive-recommended-plugins',
				[ $this, 'render_recommended_plugins_page' ]
			);
		}
	}

	private
	function add_main_menu_page(
		$title, $submenu_title, $view
	) {
		$page = add_menu_page( __( 'Integrate Google Drive', 'integrate-google-drive' ), __( 'Google Drive', 'integrate-google-drive' ), 'read',
			'integrate-google-drive', $view, IGD_ASSETS . '/images/drive.png', 30 );
		add_submenu_page( 'integrate-google-drive', $title, $submenu_title, 'read', 'integrate-google-drive' );

		return $page;
	}

	private
	function add_submenu_page(
		$title, $submenu_title, $view, $slug, $priority = 90
	) {
		$slug = str_replace( '_', '-', $slug );

		return add_submenu_page( 'integrate-google-drive', $title, $submenu_title, 'read', 'integrate-google-drive-' . $slug, $view, $priority );
	}


	public
	function render_recommended_plugins_page() {
		include IGD_INCLUDES . '/views/recommended-plugins.php';
	}

	public
	function lost_authorization_notice() {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$accounts = Account::instance()->get_accounts();

		if ( ! empty( $accounts ) ) {
			foreach ( $accounts as $id => $account ) {

				if ( ! empty( $account['lost'] ) ) {

					$msg = sprintf( '<div class="flex items-center"> <strong>Integrate Google Drive</strong> lost authorization for account <strong>%s</strong>. <a class="button" href="%s">Refresh</a></div>',
						$account['email'], admin_url( 'admin.php?page=integrate-google-drive-settings' ) );

					igd()->add_notice( 'error igd-lost-auth-notice', $msg );
				}
			}
		}

	}

	public
	function render_getting_started_page() {
		include_once IGD_INCLUDES . '/views/getting-started/index.php';
	}

	public
	function render_settings_page() { ?>
        <div id="igd-settings"></div>
	<?php }

	public
	function get_pages() {
		return array_filter( $this->pages );
	}

	/**
	 * @return Admin|null
	 */
	public
	static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}

Admin::instance();